Index: FL/Fl_File_Icon.H
===================================================================
--- FL/Fl_File_Icon.H	(revision 12885)
+++ FL/Fl_File_Icon.H	(working copy)
@@ -35,7 +35,8 @@
 
 #  define FL_ICON_COLOR (Fl_Color)0xffffffff	/**< icon color [background?]*/
 
-
+class Fl_Shared_Image;
+class Fl_RGB_Image;
 //
 // Fl_File_Icon class...
 //
@@ -53,6 +54,7 @@
   int		num_data_;	// Number of data elements
   int		alloc_data_;	// Number of allocated elements
   short		*data_;		// Icon data
+  Fl_Shared_Image *shared_data_; // Shared image data
 
   public:
 
@@ -119,6 +121,8 @@
   void		load(const char *f);
   int		load_fti(const char *fti);
   int		load_image(const char *i);
+  int		load_shared_image(Fl_Shared_Image *img);
+  int		load_rgb_image(Fl_RGB_Image *rgb);
 
   /** Returns next file icon object. See Fl_File_Icon::first() */
   Fl_File_Icon	*next() { return (next_); }
Index: src/Fl_File_Icon.cxx
===================================================================
--- src/Fl_File_Icon.cxx	(revision 12885)
+++ src/Fl_File_Icon.cxx	(working copy)
@@ -39,6 +39,7 @@
 #include <FL/Fl.H>
 #include <FL/Fl_System_Driver.H>
 #include <FL/Fl_File_Icon.H>
+#include <FL/Fl_Shared_Image.H>
 #include <FL/Fl_Widget.H>
 #include <FL/fl_draw.H>
 #include <FL/filename.H>
@@ -62,6 +63,8 @@
 			   int        nd,	/* I - Number of data values */
 			   short      *d)	/* I - Data values */
 {
+  shared_data_ = 0;
+
   // Initialize the pattern and type...
   pattern_ = p;
   type_    = t;
@@ -112,6 +115,8 @@
   // Free any memory used...
   if (alloc_data_)
     free(data_);
+  if (shared_data_)
+    shared_data_->release();
 }
 
 
Index: src/Fl_File_Icon2.cxx
===================================================================
--- src/Fl_File_Icon2.cxx	(revision 12885)
+++ src/Fl_File_Icon2.cxx	(working copy)
@@ -23,6 +23,8 @@
 //   Fl_File_Icon::load()              - Load an icon file...
 //   Fl_File_Icon::load_fti()          - Load an SGI-format FTI file...
 //   Fl_File_Icon::load_image()        - Load an image icon file...
+//   Fl_File_Icon::load_shared_image() - Load from an Fl_Shared_Image pointer.
+//   Fl_File_Icon::load_rgb_image()    - Load an RGB image.
 //   Fl_File_Icon::load_system_icons() - Load the standard system icons/filetypes.
 //   load_kde_icons()                  - Load KDE icon files.
 //   load_kde_mimelnk()                - Load a KDE "mimelnk" file.
@@ -318,10 +320,21 @@
 */
 int Fl_File_Icon::load_image(const char *ifile)	// I - File to read from
 {
-  Fl_Shared_Image	*img;		// Image file
+  if (shared_data_) return -1;
 
+  shared_data_ = Fl_Shared_Image::get(ifile);
+  return load_shared_image(shared_data_);
+}
 
-  img = Fl_Shared_Image::get(ifile);
+int Fl_File_Icon::load_rgb_image(Fl_RGB_Image *rgb)
+{
+  shared_data_ = Fl_Shared_Image::get(rgb, 1);
+  return load_shared_image(shared_data_);
+}
+
+
+int Fl_File_Icon::load_shared_image(Fl_Shared_Image *img)
+{
   if (!img || !img->count() || !img->w() || !img->h()) return -1;
 
   if (img->count() == 1) {
